<?php
namespace App\Service;

use App\Exception;

class IntegrationService {
    
    private $util;
    private $productService;
    private $integrationFactory;
    private $branchFactory;
    private $orderFactory;
    private $paymentFactory;

    public function __construct (\App\Util\Util $util, 
                                 \App\Service\ProductService $productService, 
                                 \App\Factory\BranchFactory $branchFactory, 
                                 \App\Factory\IntegrationFactory $integrationFactory, 
                                 \App\Factory\OrderFactory $orderFactory, 
                                 \App\Factory\PaymentFactory $paymentFactory) {

        $this->util               = $util;
        $this->productService     = $productService;
        $this->integrationFactory = $integrationFactory;
        $this->branchFactory      = $branchFactory;
        $this->orderFactory       = $orderFactory;
        $this->paymentFactory     = $paymentFactory;
    }
    
    public function generalParameters ($branch, $redirectStore, $merchantConfiguration) {

        $vendor  = $this->branchFactory->getStandardVendor($branch);
        $cashier = $this->branchFactory->getStandardCashier($branch, $vendor);

        if($redirectStore) {
            if(empty($merchantConfiguration['CDLOJA'])) {
                throw new Exception\ParamsException("Não existe um código de loja associado ao restaurante <b>{$merchantConfiguration['IDTOKEN']}</b> iFood.");
            }

            if(empty($merchantConfiguration['CDCAIXA'])) {
                throw new Exception\ParamsException("Não existe um código de caixa associado ao restaurante <b>{$merchantConfiguration['IDTOKEN']}</b> iFood.");
            }

            $store   = $merchantConfiguration['CDLOJA'];
            $cashier = [
                'CDCAIXA'    => $merchantConfiguration['CDCAIXA'],
                'CDOPERADOR' => $cashier['CDOPERADOR']
            ];
        } else {
            $store = $this->branchFactory->getStandardStore($branch, $vendor);
        }

        $idControlProd = $this->branchFactory->getProductControlId($branch, $store);
        $state         = $this->branchFactory->getBranchState($branch);
        
        return [
            'CDFILIAL'     => $branch,
            'CDVENDEDOR'   => $vendor,
            'CDLOJA'       => $store,
            'CDCAIXA'      => $cashier["CDCAIXA"],
            'CDOPERADOR'   => $cashier["CDOPERADOR"],
            'IDCONTROPROD' => $idControlProd,
            'SGESTADO'     => $state
        ];
    }
    
    public function deliveryRules($cdFilial, $merchantId) {
        $merchantConfiguration = $this->integrationFactory->getMerchantConfiguration($cdFilial, $merchantId);

        if(empty($merchantConfiguration)) {
            throw new Exception\ParamsException("
                Não existe um merchantId atrelado a filial {$cdFilial}.<br>
                Para obter o merchantId desta filial, entre em contato com o suporte do iFood.<br>
                Para inserir o merchantId, verifique a seção 'Configurando o Token das Filiais', no manual da integração.");
        }

        return [
            'merchantConfiguration' => $merchantConfiguration,
            'removeDeliveryFee'     => ($merchantConfiguration['IDRETTXENTR'] == "S")
        ];
    }
    
    public function validateCashierSinc($filial) {
        $caixaSinc = $this->orderFactory->fetchIsCashierSinc($filial);

        if(!isset($caixaSinc) || empty($caixaSinc)){
            throw new Exception\ParamsException("
                Não existe um caixa sincronizador atrelado a filial {$filial}.
                Para habilitar sincronização de caixa, verifique a seção 'Problemas Frequentes', no manual da integração.");
        }
    }
    
    public function openOrderBill($generalParams, $order, $consumer, $deliveryRules) {
        
        $pizzaOddFlavor = false;
        
        if(empty($generalParams["CDOPERADOR"])) {
            throw new Exception\ParamsException("
                Não existe um operador padrão atrelado ao vendedor padrão {$generalParams['CDVENDEDOR']} da filial {$generalParams['CDFILIAL']}.
                Para atrelar o operador padrão ao vendedor padrão desta filial, verifique a seção 'Problemas Frequentes', no manual da integração.");
        } 
        
        $operator = $this->orderFactory->validateOperator($generalParams);
        if(empty($operator)) {
            throw new Exception\ParamsException("
                O operador {$generalParams['CDOPERADOR']} não está associado a filial {$generalParams['CDFILIAL']}.<br>
                Para atrelar o operador a esta filial, verifique a seção 'Problemas Frequentes', no manual da integração.");
        }

        $isDuplicated = $this->orderFactory->isDuplicated($generalParams, $order);
        if($isDuplicated) {
            throw new Exception\IntegrationException("O pedido {$order['displayId']} - {$order['id']} já foi integrado pelo sistema.");
        }            

        $totalBenefits                  = 0.0;
        $voucherValue                   = 0.0;
        $productsDiscounts              = [];
        $orderHasCombinedProd           = $this->productService->checkForCombinedProducts($order['items']);

        if(isset($order['benefits']) && !empty($order['benefits'])) {
            $benefits = $order['benefits'];
            foreach($benefits as $benefit) {
    
                if($benefit['target'] == "CART") {
                    foreach($benefit['sponsorshipValues'] as $sponsorshipValue) {
                        if(($sponsorshipValue['name'] === "IFOOD") && ($sponsorshipValue['value'] != 0)) {
                            $voucherValue += $sponsorshipValue['value'];
                        }
                        
                        if(($sponsorshipValue['name'] === "EXTERNAL") && ($sponsorshipValue['value'] != 0)) {
                            $voucherValue += $sponsorshipValue['value'];
                        }
        
                        if(($sponsorshipValue['name'] === "MERCHANT") && ($sponsorshipValue['value'] != 0)) {
                            $totalBenefits += $sponsorshipValue['value'];
                        }
                    }
                }

                if(($benefit['target'] == "ITEM") || ($benefit['target'] == "PROGRESSIVE_DISCOUNT_ITEM")) {
                    foreach($benefit['sponsorshipValues'] as $sponsorshipValue) {
                        if(($sponsorshipValue['name'] === "IFOOD") && ($sponsorshipValue['value'] != 0)) {
                            $voucherValue += $sponsorshipValue['value'];
                        }
                        
                        if(($sponsorshipValue['name'] === "EXTERNAL") && ($sponsorshipValue['value'] != 0)) {
                            $voucherValue += $sponsorshipValue['value'];
                        }
        
                        if(($sponsorshipValue['name'] === "MERCHANT") && ($sponsorshipValue['value'] != 0)) {
                            if($orderHasCombinedProd){
                                $totalBenefits += $sponsorshipValue['value'];
                            } else if(!empty($productsDiscounts[$benefit['targetId']])){
                                foreach($order['items'] as $orderProduct){
                                    if($orderProduct['index'] == $benefit['targetId']){
                                        if(isset($orderProduct["options"]) && !empty($orderProduct["options"])){
                                            $productsDiscounts[$benefit['targetId']] = bcadd($productsDiscounts[$benefit['targetId']], bcdiv($sponsorshipValue['value'], $orderProduct['quantity'], 2), 2);
                                            $productsDiscountsLeftover = bcadd($productsDiscountsLeftover, bcsub($sponsorshipValue['value'], bcmul($productsDiscounts[$benefit['targetId']], $orderProduct['quantity'], 2), 2), 2);
                                        } else{
                                            $productsDiscounts[$benefit['targetId']] += $sponsorshipValue['value'];
                                            $productsDiscountsLeftover = 0;
                                        }
                                    } else if(isset($orderProduct["options"]) && !empty($orderProduct["options"])){
                                        foreach($orderProduct['options'] as $orderProductObs){
                                            if($orderProductObs['index'] == $benefit['targetId']){
                                                $productsDiscounts[$benefit['targetId']] = bcadd($productsDiscounts[$benefit['targetId']], bcdiv($sponsorshipValue['value'], $orderProductObs['quantity'], 2), 2);
                                                $productsDiscountsLeftover = bcadd($productsDiscountsLeftover, bcsub($sponsorshipValue['value'], bcmul($productsDiscounts[$benefit['targetId']], $orderProductObs['quantity'], 2), 2), 2);
                                            }
                                        }
                                    }
                                }
                            } else{
                                foreach($order['items'] as $orderProduct){
                                    if($orderProduct['index'] == $benefit['targetId']){
                                        if(isset($orderProduct["options"]) && !empty($orderProduct["options"])){
                                            $productsDiscounts[$benefit['targetId']] = bcdiv($sponsorshipValue['value'], $orderProduct['quantity'], 2);
                                            $productsDiscountsLeftover = bcsub($sponsorshipValue['value'], bcmul($productsDiscounts[$benefit['targetId']], $orderProduct['quantity'], 2), 2);
                                        } else{
                                            $productsDiscounts[$benefit['targetId']] = $sponsorshipValue['value'];
                                            $productsDiscountsLeftover = 0;
                                        }
                                    } else if(isset($orderProduct["options"]) && !empty($orderProduct["options"])){
                                        foreach($orderProduct['options'] as $orderProductObs){
                                            if($orderProductObs['index'] == $benefit['targetId']){
                                                $productsDiscounts[$benefit['targetId']] = bcdiv($sponsorshipValue['value'], $orderProductObs['quantity'], 2);
                                                $productsDiscountsLeftover = bcsub($sponsorshipValue['value'], bcmul($productsDiscounts[$benefit['targetId']], $orderProductObs['quantity'], 2), 2);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                
                $merchantDeliveryFeeDiscount = false;

                if($benefit['target'] == "DELIVERY_FEE") {
                    foreach($benefit['sponsorshipValues'] as $sponsorshipValue) {
                        if(($sponsorshipValue['name'] === "IFOOD") && ($sponsorshipValue['value'] != 0)) {
                            $voucherValue += $sponsorshipValue['value'];
                        }
                        if(($sponsorshipValue['name'] === "EXTERNAL") && ($sponsorshipValue['value'] != 0)) {
                            $voucherValue += $sponsorshipValue['value'];
                        }
        
                        if(($sponsorshipValue['name'] === "MERCHANT") && ($sponsorshipValue['value'] != 0)) {
                            $order["total"]["deliveryFee"] = ($order["total"]["deliveryFee"] - $sponsorshipValue['value']);
                            $merchantDeliveryFeeDiscount = true;
                        }
                    }
                }
            }
        }

        //------------------ Inclui pagamento realizado pelo iFood ao restaurante ------------------
        if($voucherValue != 0.0) {
            array_push($order["payments"]["methods"], [
                "name"    => "VOUCHER_IFOOD",
                "method"  => "VOUCHER",
                "value"   => $voucherValue, 
                "prepaid" => false        
            ]);
        }
        
        $sale = $this->util->newCode("VENDAREST".$generalParams["CDFILIAL"], 10);
        $comd = $this->util->newCode("COMANDAVEN".$generalParams["CDFILIAL"], 10);
        
        $order["NRVENDAREST"] = $sale;
        $order["NRCOMANDA"]   = $comd; 
        $order['NRCOMANDAEXT'] = $order['displayId'];

        if(isset($order["schedule"]) && !empty($order["schedule"])) {
            $utcTimezone   = new \DateTimeZone("UTC");
            $orderDateTime = new \DateTime($order["schedule"]["deliveryDateTimeStart"], $utcTimezone);
            $storeTimezone = new \DateTimeZone($this->util->getCurrentTimezone($generalParams["SGESTADO"]));
            
            $orderDateTime->setTimezone($storeTimezone);
            
            $order["DTHRAGENDADA"] = date_format($orderDateTime, "d/m/Y H:i:s");
            $order["IDSTCOMANDA"]  = "A";
            $order["IDSINCAGENDA"] = "N";
        } else {
            $order["DTHRAGENDADA"] = null;
            $order["IDSTCOMANDA"]  = $generalParams["IDCONTROPROD"] === "S" ? "1" : "3";
            $order["IDSINCAGENDA"] = "S";
        }

        $this->orderFactory->insertSale($sale, $generalParams, $order, $consumer);
        
        // Observação de pagamento online
        $generalParams['DSOBSCOMANDA'] = "";
        if(!empty($order["payments"]["methods"])) {
            foreach ($order["payments"]["methods"] as $payment) {
                if(isset($payment['prepaid']) && $payment['prepaid'] && isset($payment['value'])) {
                    $generalParams['DSOBSCOMANDA'] .= "PAGO ONLINE (VALOR: {$payment['value']}) ";
                }
            }            
        }

        if(!empty($order['customer']['phone']['localizer'])) {
            $generalParams['DSOBSCOMANDA'] .= "ID: " . $order['customer']['phone']['localizer'];
        }
        
        if(isset($order['schedule'])) {
            $utcTimezone   = new \DateTimeZone("UTC");
            $orderDateTime = new \DateTime($order["schedule"]["deliveryDateTimeStart"], $utcTimezone);
            $storeTimezone = new \DateTimeZone($this->util->getCurrentTimezone($generalParams["SGESTADO"]));

            $orderDateTime->setTimezone($storeTimezone);

            $generalParams['DSOBSCOMANDA'] .= " - AGENDADO PARA " . date_format($orderDateTime, "d/m/Y H:i:s");
        }
        
        if(isset($order['indoor'])){
            //varificar campo 'extraInfo' se possui informação necessária
            $generalParams['DSOBSCOMANDA'] .= " - ENTREGAR NA " . strtoupper($order['extraInfo']);
        }

        if(!empty($order['delivery']['pickupCode'])) {
            $generalParams['DSOBSCOMANDA'] .= " - COD. P/ CANCELAMENTO: " . $order['delivery']['pickupCode'];
        }

        //Regra de entregador terceirizado
        $removeDeliveryFee = $deliveryRules['removeDeliveryFee'];
        $generalParams['deliveredBy'] = 'N';

        if(isset($order["delivery"]["deliveredBy"]) && ($order["delivery"]["deliveredBy"] == "IFOOD")) {
            $removeDeliveryFee = true;
            $generalParams['deliveredBy'] = 'F';
        } else if(isset($order["delivery"]["deliveredBy"]) && ($order["delivery"]["deliveredBy"] == "MERCHANT")){
            $generalParams['deliveredBy'] = 'M';
            if($merchantDeliveryFeeDiscount == true){
                $removeDeliveryFee = false;
            }
        }

        $this->orderFactory->insertSaleCommand($generalParams, $order, $consumer, $removeDeliveryFee);

        //MOVCAIXADLV
        if(!empty($order['additionalFees'])) {
            $addFeesFlag = false;
            $addFees = 0.00;
            /*
            foreach($order['additionalFees'] as $additionalFee){
                $addFees += $additionalFee['value'];
            }
            */
            $addFees = isset($order['total']['additionalFees']) ? $order['total']['additionalFees'] : 0;
            if(isset($additionalFee)) {
                unset($additionalFee);
            }

            foreach ($order["payments"]["methods"] as &$payment) {
                if((!$addFeesFlag) && (($payment['value'] - $addFees) >= 0.01)) {
                    $payment['value'] -= $addFees;
                    $addFeesFlag = true;
                }
            }

            if(isset($payment)) {
                unset($payment);
            }
        }

        $change = 0;
        $subtractedFee = false;

        $moneyCode = $this->paymentFactory->getMoneyReceiptCode();
        
        $totalPayment = 0;
        foreach ($order["payments"]["methods"] as $payment) {
            $totalPayment += $payment["value"];
        }
        
        //para pedido zerado todos os produtos são quantificados com 1 centavo por unidade
        if($totalPayment == 0) {
            //trata pagamento minimo possivel
            $minPayment = 0.00;
            
            foreach ($order["items"] as $item) {
                if($item["price"] != 0) {
                    for ($a = 0 ; $a < $item["quantity"] ; $a++) {
                        $minPayment += 0.01;
                    }
                }
                
                if(isset($item['options']) && !empty($item['options'])){
                    foreach($item['options'] as $subItem){
                        if($subItem["price"] != 0) {
                            for ($b = 0 ; $b < $subItem["quantity"] ; $b++) {
                                $minPayment += 0.01;
                            }
                        }
                    }
                }
            }
            
            $totalBenefits -= $minPayment;
            
            //trata tipo de recebimento unico em dinheiro
            $order["NRSEQMOVDLV"] = $this->util->newCode("MOVCAIXADLV".$generalParams["CDFILIAL"].$sale, 10);
            $order["CDTIPORECE"]  = $moneyCode;
            $paymentValue = $minPayment;
        
            $this->orderFactory->insertCashMoviment($order, $paymentValue, $generalParams, $consumer);
        } else {
            
            foreach ($order["payments"]["methods"] as $payment) {
                if(isset($payment["method"]) && !empty($payment["method"])){
                    if($payment["method"] === "CASH") {
                        $receiptType = $moneyCode;
                        if(isset($order["delivery"]["deliveredBy"]) && $order["delivery"]["deliveredBy"] === "IFOOD"){
                            $paymentCode = "ONLINE_IFOOD";
                            $receiptType = $this->paymentFactory->getReceiptType($generalParams["CDFILIAL"], $paymentCode);
                        }
                    } else if ((isset($payment["method"])) && ($payment["method"] === "VOUCHER")) {
                        $paymentCode = $payment["name"];
                        $receiptType = $this->paymentFactory->getReceiptType($generalParams["CDFILIAL"], $paymentCode);
                    } else {
                        if(isset($payment["card"]) && isset($payment["card"]["brand"])) {
                            $paymentCode = $payment["type"].'_'.$payment["card"]["brand"];
                            $receiptType = $this->paymentFactory->getReceiptType($generalParams["CDFILIAL"], $paymentCode, false);
                            if(empty($receiptType)) {
                                $paymentCode = $payment["type"].'_IFOOD';
                                $receiptType = $this->paymentFactory->getReceiptType($generalParams["CDFILIAL"], $paymentCode);
                            }
                        } else if (($payment["type"] === "ONLINE") && ($payment["method"] === "OTHER")) {
                            $paymentCode = $payment["type"].'_IFOOD';
                            $receiptType = $this->paymentFactory->getReceiptType($generalParams["CDFILIAL"], $paymentCode);
                        } else {
                            throw new \Exception("Forma de pagamento não suportada");
                        }
                    }
                }
                
                $order["NRSEQMOVDLV"] = $this->util->newCode("MOVCAIXADLV".$generalParams["CDFILIAL"].$sale, 10);
                $order["CDTIPORECE"]  = $receiptType;
    
                $paymentValue = $payment["value"];
                if(($receiptType == $moneyCode) && (isset($payment["cash"])) && (isset($payment["cash"]["changeFor"])) && (!empty($payment["cash"]["changeFor"]))) {
                    $change       = bcsub($payment["cash"]["changeFor"], $payment["value"], 2);
                    $paymentValue = $payment["cash"]["changeFor"];
                }
    
                //Abate a taxa de entrega dos recebimentos
                if($removeDeliveryFee && $subtractedFee === false) {
                    if(($paymentValue - $order["total"]["deliveryFee"]) >= 0.01) {
                        $paymentValue -= $order["total"]["deliveryFee"];
                        $subtractedFee = true;
                    }
                }
            
                $this->orderFactory->insertCashMoviment($order, $paymentValue, $generalParams, $consumer);
            }
        }
        
        if($change > 0) {
            $movCaixa = $this->util->newCode("MOVCAIXADLV".$generalParams["CDFILIAL"].$sale, 10);
            $this->orderFactory->insertChange($order, $change, $generalParams, $consumer, $movCaixa, $moneyCode);
        }

        $generalParams["NRSEQPRODCUP"] = $this->util->newCode("ITCOMANDAVEN".$generalParams["CDFILIAL"], 10);
        
        $products = [];

        $ITCOMANDAVEN_SEQ = 0;

        foreach ($order["items"] as $item) {
            $processedProduct = $this->productService->processProduct($generalParams, $item, $order, $ITCOMANDAVEN_SEQ);
            
            foreach ($processedProduct as $product){
                if(isset($product["PIZZAODDFLAVOR"]) && $product["PIZZAODDFLAVOR"] == true){
                    $pizzaOddFlavor = true;
                }
            }
            
            
            if (is_array($processedProduct) && !isset($processedProduct["CDPRODUTO"]))
                $products = array_merge($products, $processedProduct);
            else
                array_push($products, $processedProduct);
        }
        
        /*if($this->productService->getHasCombinedProduct()){
            $this->productService->logCombinedError($generalParams['CDFILIAL'], $generalParams['CDCAIXA'], $generalParams['CDLOJA'], $order);
        }*/
        if(empty($products)) {
            throw new \Exception("
                Ocorreu uma falha na validação de produtos.<br>
                Contate a equipe de suporte da Teknisa.", 187);
        }

        // Ratear desconto
        if($totalBenefits != 0.0) {
            $this->shareDiscount($totalBenefits, $products, $productsDiscounts);
        } else{
            $leftoverUsed = false;
            
            foreach($products as &$prodToCheckDisc){
                if(!empty($productsDiscounts[$prodToCheckDisc["INDEX"]])){
                    if(bccomp(bcmul($prodToCheckDisc["VRPRECCOMVEN"], $prodToCheckDisc["QTPRODCOMVEN"], 2), bcadd($prodToCheckDisc["VRDESCCOMVEN"], $productsDiscounts[$prodToCheckDisc["INDEX"]], 2), 2) == 1){
                        $prodToCheckDisc["VRDESCCOMVEN"] = bcadd($prodToCheckDisc["VRDESCCOMVEN"], $productsDiscounts[$prodToCheckDisc["INDEX"]], 2);
                        if(!$leftoverUsed && (bccomp(bcsub(bcmul($prodToCheckDisc["VRPRECCOMVEN"], $prodToCheckDisc["QTPRODCOMVEN"], 2), bcadd($prodToCheckDisc["VRDESCCOMVEN"], $productsDiscounts[$prodToCheckDisc["INDEX"]], 2), 2), $productsDiscountsLeftover, 2) == 1)){
                            $prodToCheckDisc["VRDESCCOMVEN"] = bcadd($prodToCheckDisc["VRDESCCOMVEN"], $productsDiscountsLeftover, 2);
                            $leftoverUsed = true;
                        }
                    } else{
                        throw new Exception\MenuException(
                            "Descontos ultrapassam valor do produto."
                        );
                    }
                }
            }
        }



        ////////////////////////////////////////////////////////////////

        $consumerName = $order["customer"]["name"];
                    
        foreach ($products as $product) {
            
            $unityTotalValue = bcmul($product['VRPRECCOMVEN'], $product['QTPRODCOMVEN'], 2);
            $valorProduto    = number_format($unityTotalValue, 2, ',', '');
            $valorDesconto   = number_format($product['VRDESCCOMVEN'], 2, ',', '');

            if(empty($product['VRPRECCOMVEN']) || $product['VRPRECCOMVEN'] < 0) {
                throw new \Exception("
                    O produto {$product['NMPRODUTOIFOOD']} possui preço inferior a R$ 0,01 centavo.<br>
                    Código do produto: {$product['CDPRODUTO']}.<br>
                    Valor do produto: R$ {$valorProduto}.<br>
                    Caso o produto seja um combo, em que o item pai não possui preço, é necessário excluir o código do produto do item pai.<br>
                    A exclusão do código externo deve ser feito no portal iFood.");
            }
            
            if($product['VRDESCCOMVEN'] >= $unityTotalValue) {
                throw new \Exception("
                    O produto {$product['NMPRODUTOIFOOD']} possui desconto superior ao valor de venda.<br>
                    Valor do produto: R$ {$valorProduto}.<br>
                    Desconto aplicado: R$ {$valorDesconto}.<br>
                    Gentileza entrar em contato com a equipe de suporte da Teknisa.");
            }
            
            $this->productService->insertProduct(array_merge($product, $generalParams), $consumerName);
            
            //Observações da OBSITCOMANDAVEN
            if(!empty($product['ITEM_OBSERVATION'])) {
                
                $product['ITEM_OBSERVATION'] = array_unique($product['ITEM_OBSERVATION'], SORT_REGULAR);

                foreach($product['ITEM_OBSERVATION'] as $itemObservation) {
                    $this->productService->insertObservation($itemObservation);
                }                
            }
        }

        return [
            'CDFILIAL'       => $generalParams["CDFILIAL"],
            'NRCOMANDA'      => $order["NRCOMANDA"],
            'NRCOMANDAEXT'   => $order["displayId"],
            'NRVENDAREST'    => $order["NRVENDAREST"],
            'PIZZAODDFLAVOR' => $pizzaOddFlavor
        ];
    }
    
    public function shareDiscount($initialDiscount, &$products, $productsDiscounts) {

        $idxToApplyDiscount   = [];
        $totalDiscount        = $initialDiscount;
        $discountLeft         = $initialDiscount;
        $productsTotalValue   = "0.00";
        $discountGivenShare   = "0.00";
        $maxValueItem         = "0.00";
        $maxValueIdx          = -1;
        
        foreach($products as &$product) {
            $product['VRDESCCOMVEN']  = 0;
            $product['discountShare'] = "0.00";
        }
        
        if(isset($product))
            unset($product);

        foreach($products as $i => $product) {
            if((bccomp($product["VRPRECCOMVEN"], "0.01", 2) == 1)) { 
                $unityTotalValue = bcmul($product['VRPRECCOMVEN'], $product['QTPRODCOMVEN'], 2);
                $productsTotalValue = bcadd($productsTotalValue, $unityTotalValue, 2);
                array_push($idxToApplyDiscount, $i);
            }
        }
        
        foreach($productsDiscounts as $productDiscount){
            $productsTotalValue = bcsub($productsTotalValue, $productDiscount, 2);
        }

        foreach($idxToApplyDiscount as $idxWithDiscount) {
            $products[$idxWithDiscount]['TOTALVALUE'] = bcmul($products[$idxWithDiscount]['VRPRECCOMVEN'], $products[$idxWithDiscount]['QTPRODCOMVEN'], 2);
            
            if(!empty($productsDiscounts[$products[$idxWithDiscount]['INDEX']])){
                $products[$idxWithDiscount]['TOTALVALUE'] = bcsub($products[$idxWithDiscount]['TOTALVALUE'], $productsDiscounts[$products[$idxWithDiscount]['INDEX']], 2);
            }
            
            $discountShare = bcdiv((bcmul($products[$idxWithDiscount]['TOTALVALUE'], "100.00", 2)), $productsTotalValue, 2);
            $products[$idxWithDiscount]["discountShare"] = $discountShare;

            $discountGivenShare = bcadd($discountGivenShare, $discountShare, 2);
        }

        foreach($products as $i => $product) {
            if(bccomp($product['discountShare'], $maxValueItem, 2) == 1) {
                $maxValueIdx  = $i;
                $maxValueItem = $product['discountShare'];
            }            
        }

        if($maxValueIdx != -1) {
            $products[$maxValueIdx]["discountShare"] = bcadd($products[$maxValueIdx]["discountShare"], bcsub("100.00", $discountGivenShare, 2), 2);
        }

        foreach($idxToApplyDiscount as $idxWithDiscount) {
        
            $itemDiscount = bcdiv((bcmul($products[$idxWithDiscount]["discountShare"], $totalDiscount, 2)), "100.00", 2);
            
            $products[$idxWithDiscount]["VRDESCCOMVEN"] = $itemDiscount;

            $discountLeft = bcsub($discountLeft, $itemDiscount, 2);
            
            if(!empty($productsDiscounts[$products[$idxWithDiscount]['INDEX']])){
                $products[$idxWithDiscount]["VRDESCCOMVEN"] = bcadd($products[$idxWithDiscount]["VRDESCCOMVEN"], $productsDiscounts[$products[$idxWithDiscount]['INDEX']], 2);
            }
        }

        if($maxValueIdx != -1) {
            $products[$maxValueIdx]["VRDESCCOMVEN"] = bcadd($products[$maxValueIdx]["VRDESCCOMVEN"], $discountLeft, 2);
            $discountLeft = "0.00";
        }
        
        $leftDiscount = "0.00";
        
        foreach($products as &$product) {
            $unityTotalValue = bcmul($product['VRPRECCOMVEN'], $product['QTPRODCOMVEN'], 2);
            if( (bccomp($product['VRDESCCOMVEN'], $unityTotalValue, 2) == 1) || (bccomp($product['VRDESCCOMVEN'], $unityTotalValue, 2) == 0) )  {
                $leftDiscount = bcadd($leftDiscount, bcsub($product['VRDESCCOMVEN'], $product['VRPRECCOMVEN'], 2), 2);
                $leftDiscount = bcadd($leftDiscount, "0.01", 2);
                
                $product['VRDESCCOMVEN'] = bcsub($product['VRDESCCOMVEN'], $leftDiscount, 2);
            }
        }
        
        if(isset($product))
            unset($product);

        foreach($products as &$product){
            $unityTotalValue = bcmul($product['VRPRECCOMVEN'], $product['QTPRODCOMVEN'], 2);
            $maxAddDisc = (bcsub(bcsub($unityTotalValue, "0.01", 2), $product['VRDESCCOMVEN'], 2));
            if($maxAddDisc >= $leftDiscount) {
                $addDisc = $leftDiscount;
            } else{
                $addDisc = $maxAddDisc;
            }

            $product['VRDESCCOMVEN'] = bcadd($product['VRDESCCOMVEN'], $addDisc, 2);
            $leftDiscount = bcsub($leftDiscount, $addDisc, 2);
        }

        if(isset($product))
            unset($product);
        
        return $products;
    }
    
    public function cancelOrder($orderReference) {
        $orderDetails = $this->integrationFactory->getOrderForCancel($orderReference);
        
        if(empty($orderDetails))
           return 0;
        
        $this->integrationFactory->cancelOrder($orderDetails);
        $this->integrationFactory->syncOrder($orderDetails);
        return $orderDetails;
    }
    
    public function validateOrder($cdFilial, $nrVendaRest, $pizzaOddFlavor) {
        $this->orderFactory->validateOrder($cdFilial, $nrVendaRest, $pizzaOddFlavor);
    }
    
    public function disableIntegration() {
        $this->integrationFactory->disableIntegration();
    }

    public function getConfiguration() {
        return $this->integrationFactory->getConfiguration();
    }
    
    public function updateAccessToken($newToken, $time) {
        return $this->integrationFactory->updateAccessToken($newToken, $time);
    }
    
}
